/*
 * Copyright (c) 2016 Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

/* Autogenerated file, please don't edit */

#include "mysqlx_error.h"

  {"ER_X_BAD_MESSAGE", ER_X_BAD_MESSAGE, ""},
  {"ER_X_CAPABILITIES_PREPARE_FAILED", ER_X_CAPABILITIES_PREPARE_FAILED, ""},
  {"ER_X_CAPABILITY_NOT_FOUND", ER_X_CAPABILITY_NOT_FOUND, ""},
  {"ER_X_INVALID_PROTOCOL_DATA", ER_X_INVALID_PROTOCOL_DATA, ""},
  {"ER_X_SERVICE_ERROR", ER_X_SERVICE_ERROR, ""},
  {"ER_X_SESSION", ER_X_SESSION, ""},
  {"ER_X_INVALID_ARGUMENT", ER_X_INVALID_ARGUMENT, ""},
  {"ER_X_MISSING_ARGUMENT", ER_X_MISSING_ARGUMENT, ""},
  {"ER_X_BAD_INSERT_DATA", ER_X_BAD_INSERT_DATA, ""},
  {"ER_X_CMD_NUM_ARGUMENTS", ER_X_CMD_NUM_ARGUMENTS, ""},
  {"ER_X_CMD_ARGUMENT_TYPE", ER_X_CMD_ARGUMENT_TYPE, ""},
  {"ER_X_CMD_ARGUMENT_VALUE", ER_X_CMD_ARGUMENT_VALUE, ""},
  {"ER_X_BAD_UPDATE_DATA", ER_X_BAD_UPDATE_DATA, ""},
  {"ER_X_BAD_TYPE_OF_UPDATE", ER_X_BAD_TYPE_OF_UPDATE, ""},
  {"ER_X_BAD_COLUMN_TO_UPDATE", ER_X_BAD_COLUMN_TO_UPDATE, ""},
  {"ER_X_BAD_MEMBER_TO_UPDATE", ER_X_BAD_MEMBER_TO_UPDATE, ""},
  {"ER_X_BAD_STATEMENT_ID", ER_X_BAD_STATEMENT_ID, ""},
  {"ER_X_BAD_CURSOR_ID", ER_X_BAD_CURSOR_ID, ""},
  {"ER_X_BAD_SCHEMA", ER_X_BAD_SCHEMA, ""},
  {"ER_X_BAD_TABLE", ER_X_BAD_TABLE, ""},
  {"ER_X_BAD_PROJECTION", ER_X_BAD_PROJECTION, ""},
  {"ER_X_DOC_ID_MISSING", ER_X_DOC_ID_MISSING, ""},
  {"ER_X_DOC_ID_DUPLICATE", ER_X_DOC_ID_DUPLICATE, ""},
  {"ER_X_DOC_REQUIRED_FIELD_MISSING", ER_X_DOC_REQUIRED_FIELD_MISSING, ""},
  {"ER_X_PROJ_BAD_KEY_NAME", ER_X_PROJ_BAD_KEY_NAME, ""},
  {"ER_X_BAD_DOC_PATH", ER_X_BAD_DOC_PATH, ""},
  {"ER_X_CURSOR_EXISTS", ER_X_CURSOR_EXISTS, ""},
  {"ER_X_EXPR_BAD_OPERATOR", ER_X_EXPR_BAD_OPERATOR, ""},
  {"ER_X_EXPR_BAD_NUM_ARGS", ER_X_EXPR_BAD_NUM_ARGS, ""},
  {"ER_X_EXPR_MISSING_ARG", ER_X_EXPR_MISSING_ARG, ""},
  {"ER_X_EXPR_BAD_TYPE_VALUE", ER_X_EXPR_BAD_TYPE_VALUE, ""},
  {"ER_X_EXPR_BAD_VALUE", ER_X_EXPR_BAD_VALUE, ""},
  {"ER_X_INVALID_COLLECTION", ER_X_INVALID_COLLECTION, ""},
  {"ER_X_INVALID_ADMIN_COMMAND", ER_X_INVALID_ADMIN_COMMAND, ""},
  {"ER_X_EXPECT_NOT_OPEN", ER_X_EXPECT_NOT_OPEN, ""},
  {"ER_X_EXPECT_FAILED", ER_X_EXPECT_FAILED, ""},
  {"ER_X_EXPECT_BAD_CONDITION", ER_X_EXPECT_BAD_CONDITION, ""},
  {"ER_X_EXPECT_BAD_CONDITION_VALUE", ER_X_EXPECT_BAD_CONDITION_VALUE, ""},
  {"ER_X_INVALID_NAMESPACE", ER_X_INVALID_NAMESPACE, ""},
  {"ER_X_BAD_NOTICE", ER_X_BAD_NOTICE, ""},
  {"ER_X_CANNOT_DISABLE_NOTICE", ER_X_CANNOT_DISABLE_NOTICE, ""},
  {"ER_X_BAD_CONFIGURATION", ER_X_BAD_CONFIGURATION, ""},
  {"ER_X_MYSQLX_ACCOUNT_MISSING_PERMISSIONS", ER_X_MYSQLX_ACCOUNT_MISSING_PERMISSIONS, ""},

